# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2023 Lina Krisztian and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Import raster with pixellimit check
"""
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except Exception:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

__license__ = "GPLv3"
__author__ = "Lina Krisztian"
__copyright__ = "Copyright 2023, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


class ImportRasterLayerPixellimitTestCase(ActiniaResourceTestCaseBase):
    location = "nc_spm_08"
    tmp_mapset = "mapset_rasterimport_pixellimit"
    endpoint = f"/locations/{location}/mapsets/{tmp_mapset}/processing_async"
    rimport_inp = "elevation"
    # import resolution with which the process should fail:
    rimport_res_fail = 0.1

    def setUp(self):
        # create new temp mapset
        super(ImportRasterLayerPixellimitTestCase, self).setUp()
        self.create_new_mapset(self.tmp_mapset, location_name=self.location)

    def tearDown(self):
        # delete mapset
        self.delete_mapset(self.tmp_mapset, location_name=self.location)
        super(ImportRasterLayerPixellimitTestCase, self).tearDown()

    def test_pixellimit_allowed(self):
        """
        Test import of raster, for which pixellimit is not reached
        and therefore allowed
        """
        raster_url = additional_external_data[self.rimport_inp]
        raster = self.rimport_inp
        process_chain = {
            "version": 1,
            "list": [
                {
                    "id": "1",
                    "module": "r.import",
                    "inputs": [
                        {
                            "param": "input",
                            "value": raster_url,
                        },
                        {
                            "param": "output",
                            "value": raster,
                        },
                    ],
                },
            ],
        }
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )
        # Import should succeed
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_pixellimit_not_allowed(self):
        """
        Test import of raster, for which pixellimit is reached
        and therefore not allowed
        """
        raster_url = additional_external_data[self.rimport_inp]
        raster = self.rimport_inp
        process_chain = {
            "version": 1,
            "list": [
                {
                    "id": "1",
                    "module": "r.import",
                    "inputs": [
                        {
                            "param": "input",
                            "value": raster_url,
                        },
                        {
                            "param": "output",
                            "value": raster,
                        },
                        {
                            "param": "resolution",
                            "value": "value",
                        },
                        {
                            "param": "resolution_value",
                            "value": f"{self.rimport_res_fail}",
                        },
                    ],
                },
            ],
        }
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )
        # Import should fail with certain message (due too high resolution)
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
        )
        self.assertTrue(
            "Processing pixel limit exceeded for raster import"
            in resp["exception"]["message"]
        )

    def test_pixellimit_importer(self):
        """
        Test import of raster with the importer
        with pixellimit not reached and therefore allowed
        """
        raster_url = additional_external_data[self.rimport_inp]
        raster = self.rimport_inp
        process_chain = {
            "version": 1,
            "list": [
                {
                    "id": "1",
                    "module": "importer",
                    "inputs": [
                        {
                            "import_descr": {
                                "source": raster_url,
                                "type": "raster",
                            },
                            "param": "map",
                            "value": raster,
                        },
                    ],
                },
            ],
        }
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )
        # Import should succeed
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Keycloak user management

Possible TODOs: add list_all_users, delete, create_user (, exists).
        In this case, keycloak admin console would be needed and actinia would
        need to store keycloak admin credentials!
"""
from keycloak import KeycloakOpenID

from actinia_core.core.common.user_base import (
    ActiniaUserBase,
)
from actinia_core.core.logging_interface import log
from actinia_core.core.common.config import global_config

__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


def create_user_from_tokeninfo(token_info):
    """
    Function to create a keycloak user from the keycloak token.
    """
    attr_prefix = (
        global_config.KEYCLOAK_ATTR_PREFIX
        if global_config.KEYCLOAK_ATTR_PREFIX is not None
        else ""
    )
    user_id = token_info["preferred_username"]
    kwargs = {
        "user_role": token_info["resource_access"][
            global_config.KEYCLOAK_CLIENT_ID
        ]["roles"][0],
        "cell_limit": token_info[f"{attr_prefix}cell_limit"],
        "process_num_limit": token_info[f"{attr_prefix}process_num_limit"],
        "process_time_limit": token_info[f"{attr_prefix}process_time_limit"],
    }
    acc_ds_name = f"{attr_prefix}accessible_datasets"
    if token_info[acc_ds_name] and token_info[acc_ds_name] != "None":
        kwargs["accessible_datasets"] = token_info[acc_ds_name]
    acc_mod_name = f"{attr_prefix}accessible_modules"
    if token_info[acc_mod_name] and token_info[acc_mod_name] != "None":
        kwargs["accessible_modules"] = token_info[acc_mod_name]
    groups = list()
    for group in token_info["groups"]:
        if group.startswith(global_config.KEYCLOAK_GROUP_PREFIX):
            groups.append(
                group.replace(global_config.KEYCLOAK_GROUP_PREFIX, "")
            )
    if len(groups) == 0:
        log.error(f"Authentication ERROR: User {user_id} has no user group.")
        return None
    elif len(groups) > 1:
        log.warning(
            f"User {user_id} has more than one group, taking {groups[0]}."
        )
    kwargs["user_group"] = groups[0]
    user = ActiniaKeycloakUser(
        user_id,
        **kwargs,
    )
    user._generate_permission_dict()
    # adding group members
    if "group_members" in token_info and token_info["group_members"]:
        user.set_group_members(
            ",".join(token_info["group_members"]).split(",")
        )
    return user


class ActiniaKeycloakUser(ActiniaUserBase):
    """
    The Actinia Core keycloak user management class

    This class manages a user which is stored in keycloak
    """

    def __init__(
        self,
        user_id,
        user_group=None,
        user_role=None,
        accessible_datasets={
            "nc_spm_08": ["PERMANENT", "user1", "landsat"],
            "ECAD": ["PERMANENT"],
            "latlong_wgs84": ["PERMANENT"],
        },
        accessible_modules=global_config.MODULE_ALLOW_LIST,
        cell_limit=global_config.MAX_CELL_LIMIT,
        process_num_limit=global_config.PROCESS_NUM_LIMIT,
        process_time_limit=global_config.PROCESS_TIME_LIMT,
    ):
        if isinstance(accessible_datasets, str):
            datasets = dict()
            lm_list = accessible_datasets.split(",")
            for lm in lm_list:
                if "/" in lm:
                    location, mapset = lm.split("/")
                else:
                    location = lm
                    mapset = None
                if location not in datasets:
                    datasets[location] = []
                datasets[location].append(mapset)
        else:
            datasets = accessible_datasets
        if isinstance(accessible_modules, str):
            modules = accessible_modules.split(",")
        else:
            modules = accessible_modules
        super().__init__(
            user_id,
            user_group=user_group,
            user_role=user_role,
            accessible_datasets=datasets,
            accessible_modules=modules,
            cell_limit=cell_limit,
            process_num_limit=process_num_limit,
            process_time_limit=process_time_limit,
        )
        self.group_members = list()

    @staticmethod
    def verify_keycloak_token(token):
        keycloak_openid = KeycloakOpenID(
            server_url=global_config.KEYCLOAK_URL,
            client_id=global_config.KEYCLOAK_CLIENT_ID,
            realm_name=global_config.KEYCLOAK_REALM,
            client_secret_key=global_config.KEYCLOAK_CLIENT_SECRET_KEY,
        )
        KEYCLOAK_PUBLIC_KEY = (
            "-----BEGIN PUBLIC KEY-----\n"
            + keycloak_openid.public_key()
            + "\n-----END PUBLIC KEY-----"
        )
        options = {
            "verify_signature": True,
            "verify_aud": True,
            "verify_exp": True,
        }
        try:
            token_info = keycloak_openid.decode_token(
                token, key=KEYCLOAK_PUBLIC_KEY, options=options
            )
        except Exception:
            return None
        return create_user_from_tokeninfo(token_info)

    def set_group_members(self, group_members):
        """Set the user group_members

        Args:
            group: The user group_members

        """
        self.group_members = group_members

    def check_group_members(self, user_id):
        """Check if the user_id is in the group_members attribute.

        Args:
            user_id (str): The id (name, email, ..) of the user that must be
                           unique

        Returns:
            bool:
            Return the if the user_id is in the group_members
        """
        return user_id in self.group_members

    # def exists(self):  # rest/user_management ???
    #     """Check if the user exists
    #
    #     Returns:
    #         bool:
    #         True if the user exists, False otherwise
    #     """
    #     if self.user_id is None:
    #         return False
    #
    #     return self.db.exists(self.user_id)

    def get_role(self):
        """Return the role

        Returns:
            str:
            Return the role
        """
        return self.user_role

    def get_group(self):
        """Return the user group

        Returns:
            str:
            Return the user group
        """
        return self.user_group

    def get_credentials(self):
        """Return the user credentials as a dictionary

        Returns:
            dict:
            Return the user credentials as a dictionary
        """
        self._generate_permission_dict()
        credentials = {
            "user_id": self.user_id,
            "user_role": self.user_role,
            "user_group": self.user_group,
            "permissions": self.permissions,
        }
        return credentials

    def get_accessible_datasets(self):
        """Return a dictionary of location:mapset list entries

        Returns:
            dict:
            Return a dictionary of location:mapset list entries
        """
        self._generate_permission_dict()

        if self.permissions and "accessible_datasets" in self.permissions:
            return self.permissions["accessible_datasets"]

    def get_accessible_modules(self):
        """Return a list of all accessible modules

        Returns:
            list:
            Return a list of all accessible modules
        """
        self._generate_permission_dict()

        if self.permissions and "accessible_modules" in self.permissions:
            return self.permissions["accessible_modules"]

    def get_cell_limit(self):
        """Return the cell limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "cell_limit" in self.permissions:
            return self.permissions["cell_limit"]

    def get_process_num_limit(self):
        """Return the process number limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "process_num_limit" in self.permissions:
            return self.permissions["process_num_limit"]

    def get_process_time_limit(self):
        """Return the process time limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "process_time_limit" in self.permissions:
            return self.permissions["process_time_limit"]



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster colors management
"""

from actinia_core.processing.common.utils import try_import

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


EphemeralRasterColorsOutput = try_import(
    "actinia_core.processing.actinia_processing.ephemeral.raster_colors",
    "EphemeralRasterColorsOutput",
)

PersistentRasterColorsRules = try_import(
    "actinia_core.processing.actinia_processing.persistent.raster_colors",
    "PersistentRasterColorsRules",
)


def start_job_colors_out(*args):
    processing = EphemeralRasterColorsOutput(*args)
    processing.run()


def start_job_from_rules(*args):
    processing = PersistentRasterColorsRules(*args)
    processing.run()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2024 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async Process stdout parser test case
"""
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2024, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

PC_RUNIVAR = {
    "list": [
        {
            "id": "set_region_to_raster",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "landuse96_28m"}],
            "flags": "g",
        },
        {
            "id": "get_stats",
            "module": "r.univar",
            "inputs": [{"param": "map", "value": "landuse96_28m"}],
            "flags": "g",
        },
        {
            "id": "calculation_scalling",
            "module": "r.mapcalc",
            "inputs": [
                {
                    "param": "expression",
                    "value": "test = (landuse96_28m - get_stats::stdout::min) "
                    "/ (get_stats::stdout::max - get_stats::stdout::min)",
                }
            ],
        },
        {
            "id": "get_stats_result",
            "module": "r.univar",
            "inputs": [{"param": "map", "value": "test"}],
            "stdout": {
                "id": "stats_zonal",
                "format": "kv",
                "delimiter": "=",
            },
            "flags": "g",
        },
    ],
    "version": "1",
}

PC_GLIST = {
    "list": [
        {"id": "print_region", "module": "g.region", "flags": "p"},
        {
            "id": "g_list_rasterpattern",
            "module": "g.list",
            "inputs": [
                {"param": "type", "value": "raster"},
                {"param": "pattern", "value": "l*"},
                {"param": "separator", "value": "comma"},
            ],
        },
        {
            "id": "set_region",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "g_list_rasterpattern::stdout"}
            ],
            "flags": "g",
        },
    ],
    "version": "1",
}


class AsyncProcessStdinParameterParserTestCase(ActiniaResourceTestCaseBase):
    def test_glist_parsing(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(PC_GLIST),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertIn(
            "landuse96_28m,lsat7_2002_10",
            resp["process_log"][1]["stdout"],
            "Process log is wrong",
        )
        self.assertIn(
            "raster=landuse96_28m,lsat7_2002_10",
            resp["process_log"][2]["parameter"],
            "Process parameters are set incorrectly.",
        )

    def test_runivar_parsing(self):
        min, max = 0, 21
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(PC_RUNIVAR),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertIn(f"min={min}", resp["process_log"][1]["stdout"])
        self.assertIn(f"max={max}", resp["process_log"][1]["stdout"])
        self.assertIn(
            f"- {min}) / ({max} - {min})",
            resp["process_log"][2]["parameter"][0],
            "Process parameters (min and max) are set incorrectly.",
        )
        self.assertEqual(
            float(resp["process_results"]["stats_zonal"]["max"]), 1
        )
        self.assertEqual(
            float(resp["process_results"]["stats_zonal"]["min"]), 0
        )


if __name__ == "__main__":
    unittest.main()
